//
//  DirectoryIteratorTest.cpp
//
//  Created  by Setsu on 5/23/2010.
//  Modified by Setsu on May 30, 2013.
//  Copyright 2010-2013 RoundSquare Inc. All rights reserved.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//

//----------------------------------------
//	include
//----------------------------------------
#include <Poco/File.h>
#include <Poco/Path.h>
#include <Poco/Format.h>
#include <Poco/Exception.h>
#include <Poco/DirectoryIterator.h>
#include <Poco/LocalDateTime.h>
#include <Poco/DateTimeFormatter.h>
#include <Poco/DateTimeFormat.h>
#include <Poco/Timezone.h>

#include <string>

#include "ScopedLogMessage.h"
#include "PrepareConsoleLogger.h"

//----------------------------------------
//	ShowDirectoriesAndFiles
//----------------------------------------
void ShowDirectoriesAndFiles(ScopedLogMessage& msg, const std::string& path)
{
	Poco::DirectoryIterator dirItr(path);
	Poco::DirectoryIterator	end;
	while(dirItr != end)
	{
		std::string attr(dirItr->isDirectory() ? "d":"-");
		attr += dirItr->canRead() ? "r":"-";
		attr += dirItr->canWrite() ? "w":"-";

		Poco::LocalDateTime dateTime(dirItr->getLastModified());

		msg.Message(Poco::format("  %s %s %s"
								,	attr
								,	Poco::DateTimeFormatter::format(dateTime, Poco::DateTimeFormat::SORTABLE_FORMAT)
								,	dirItr->path()));
		if(dirItr->isDirectory())
		{
			ShowDirectoriesAndFiles(msg, dirItr->path());
		}
		++dirItr;
	}
}

//----------------------------------------
//	main
//----------------------------------------
int main(int /*argc*/, char** /*argv*/)
{
	PrepareConsoleLogger logger(Poco::Logger::ROOT, Poco::Message::PRIO_INFORMATION);

	ScopedLogMessage msg("DirectoryIteratorTest ", "start", "end");

	msg.Message("");
	msg.Message("--- create sample directories and files ---");
	const char* dirNames[] = {	"testDir1"
							,	"testDir2"
							,	"testDir3"	};
	const int kNumDir = sizeof(dirNames)/sizeof(dirNames[0]);
	const int kNumFilesInDir = 3;

	std::string nThPath[kNumDir];

	for(int i=0; i<kNumDir; ++i)
	{
		for(int j=0; j<=i; ++j)
		{
			nThPath[i] += std::string(dirNames[j]) + Poco::Path::separator();
		}

		Poco::File dir(Poco::Path(nThPath[i], Poco::Path::PATH_UNIX));
		if(0 == i && dir.exists())
		{
			try
			{
				dir.remove(true);
				msg.Message(Poco::format("  %s recursively removed", dir.path()));
			}
			catch(const Poco::Exception& exp)
			{
				msg.Message(Poco::format("  EXCEPTION [%s]", exp.displayText()));
			}
		}
		dir.createDirectories();
		msg.Message(Poco::format("  %s created", dir.path()));

		for(int k=0; k<kNumFilesInDir; ++k)
		{
			Poco::File file(Poco::Path(dir.path(), Poco::format("testFile%d_%d.txt", i+1, k)));
			file.createFile();
			msg.Message(Poco::format("    %s created", file.path()));
		}
	}

	msg.Message("");
	msg.Message("--- show the directories and files by DirectoryIterator ---");
	ShowDirectoriesAndFiles(msg, nThPath[0]);
	msg.Message("");

	return 0;
}
